/* jslint node: true, esnext: true */
"use strict";

const BaseStep = require('kronos-step').Step;
const messageFactory = require('kronos-message');

const lineParserFactory = require('stream-line-parser');
const lineTokenizerFactory = require('stream-line-tokenizer-csv');
const lineHeaderFactory = require('stream-line-header');
const tokens2objFactory = require('stream-line-tokens2obj');
const rowProcessorFactory = require('stream-object-data-processor-row');
const chunkProcessorFactory = require('stream-object-data-processor-chunk');

const DataProcessor = {
	"name": "kronos-step-passThrough",
	"description": "This step just forwards each request from its in endpoint to its out endpoint",
	"endpoints": {
		"inFileStream": {
			"in": true
		},
		"outRowStream": {
			"out": true
		}
	},

	initialize(manager, scopeReporter, name, config, props) {
		if (!config.line_parser) {
			throw new Error('ERROR: no line_parser configuration given in the config.');
		}
		if (!config.line_tokenizer) {
			throw new Error('ERROR: no line_tokenizer configuration given in the config.');
		}
		if (!config.header) {
			throw new Error('ERROR: no header configuration given in the config.');
		}
		if (!config.tokens2obj) {
			throw new Error('ERROR: no tokens2obj configuration given in the config.');
		}

		if (config.chunkProcessor) {
			if (!config.rowProcessor) {
				throw new Error("ERROR: 'chunkProcessor' config is given the 'rowProcessor' config is mandatory.");
			}
		}

		props.config = {
			value: config
		};

		// just for testing the schema of the configuration
		lineParserFactory(config.line_parser, true);
		lineTokenizerFactory(config.line_tokenizer, true);
		lineHeaderFactory(config.header, true);
		tokens2objFactory(config.tokens2obj, true);


	},

	finalize(manager, scopeReporter, stepConfiguration) {
		const self = this;
		const inEndpoint = this.endpoints.inFileStream;
		inEndpoint.receive = function (messageToClone) {
			let stream = messageToClone.payload;

			// for each request new p
			let line_parser = lineParserFactory(self.config.line_parser, true);
			let line_tokenizer = lineTokenizerFactory(self.config.line_tokenizer, true);
			let header = lineHeaderFactory(self.config.header, true);
			let tokens2obj = tokens2objFactory(self.config.tokens2obj, true);

			let rowProcessor;
			let chunkProcessor;
			if (self.config.rowProcessor) {
				rowProcessor = rowProcessorFactory(self.config.rowProcessor, true);
			}
			if (self.config.chunkProcessor) {
				chunkProcessor = chunkProcessorFactory(self.config.chunkProcessor, true);
			}

			// create a new message clone
			const message = messageFactory(undefined, messageToClone);

			if (chunkProcessor) {
				message.payload = stream.pipe(line_parser).pipe(line_tokenizer).pipe(header).pipe(tokens2obj).pipe(rowProcessor).pipe(
					chunkProcessor);
			} else if (rowProcessor) {
				message.payload = stream.pipe(line_parser).pipe(line_tokenizer).pipe(header).pipe(tokens2obj).pipe(rowProcessor);
			} else {
				message.payload = stream.pipe(line_parser).pipe(line_tokenizer).pipe(header).pipe(tokens2obj);
			}

			self.endpoints.outRowStream.send(message);
		};
	}


};

let obj = Object.assign({}, BaseStep, DataProcessor);
module.exports = obj;
